@@ -190,7 +190,7 @@ class Ecobee(object):
                   'Authorization': 'Bearer ' + self.access_token}
         params = {'format': 'json'}
         try:
-            request = requests.post(url, headers=header, params=params, data=body)
+            request = requests.post(url, headers=header, params=params, data=json.dumps(body))
         except RequestException:
             logger.warn("Error connecting to Ecobee.  Possible connectivity outage.")
             return None
@@ -200,89 +200,119 @@ class Ecobee(object):
             logger.info("Error connecting to Ecobee while attempting to %s.  "
                         "Refreshing tokens and trying again.", log_msg_action)
             if self.refresh_tokens():
-                return self.make_request(body, log_msg_action)
+                return self.make_request(json.dumps(body), log_msg_action)
             else:
                 return None
 
     def set_hvac_mode(self, index, hvac_mode):
         ''' possible hvac modes are auto, auxHeatOnly, cool, heat, off '''
-        body = ('{"selection":{"selectionType":"thermostats","selectionMatch":'
-                '"' + self.thermostats[index]['identifier'] +
-                '"},"thermostat":{"settings":{"hvacMode":"' + hvac_mode +
-                '"}}}')
+        body = {"selection": {"selectionType": "thermostats",
+                              "selectionMatch": self.thermostats[index]['identifier']},
+                              "thermostat": {
+                                  "settings": {
+                                      "hvacMode": hvac_mode
+                                  }
+                              }}
         log_msg_action = "set HVAC mode"
         return self.make_request(body, log_msg_action)
 
     def set_fan_min_on_time(self, index, fan_min_on_time):
         ''' The minimum time, in minutes, to run the fan each hour. Value from 1 to 60 '''
-        body = ('{"selection":{"selectionType":"thermostats","selectionMatch":'
-                '"' + self.thermostats[index]['identifier'] +
-                '"},"thermostat":{"settings":{"fanMinOnTime":"' + fan_min_on_time +
-                '"}}}')
+        body = {"selection": {"selectionType": "thermostats",
+                        "selectionMatch": self.thermostats[index]['identifier']},
+                        "thermostat": {
+                            "settings": {
+                                "fanMinOnTime": fan_min_on_time
+                            }
+                        }}
         log_msg_action = "set fan minimum on time."
         return self.make_request(body, log_msg_action)
 
-    def set_fan_mode(self, index, fan_mode):
+    def set_fan_mode(self, index, fan_mode, cool_temp, heat_temp, hold_type="nextTransition"):
         ''' Set fan mode. Values: auto, minontime, on '''
-        body = ('{"selection":{"selectionType":"thermostats","selectionMatch":'
-                '"' + self.thermostats[index]['identifier'] +
-                '"},"thermostat":{"settings":{"vent":"' + fan_mode +
-                '"}}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "setHold", "params": {
+                    "holyType": hold_type,
+                    "coolHoldTemp": cool_temp * 10,
+                    "heatHoldTemp": heat_temp * 10,
+                    "fan": fan_mode
+                }}]}
         log_msg_action = "set fan mode"
         return self.make_request(body, log_msg_action)
 
     def set_hold_temp(self, index, cool_temp, heat_temp,
                       hold_type="nextTransition"):
         ''' Set a hold '''
-        body = ('{"functions":[{"type":"setHold","params":{"holdType":"'
-                + hold_type + '","coolHoldTemp":"' + str(int(cool_temp * 10)) +
-                '","heatHoldTemp":"' + str(int(heat_temp * 10)) + '"}}],'
-                '"selection":{"selectionType":"thermostats","selectionMatch"'
-                ':"' + self.thermostats[index]['identifier'] + '"}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "setHold", "params": {
+                    "holyType": hold_type,
+                    "coolHoldTemp": cool_temp * 10,
+                    "heatHoldTemp": heat_temp * 10
+                }}]}
         log_msg_action = "set hold temp"
         return self.make_request(body, log_msg_action)
 
     def set_climate_hold(self, index, climate, hold_type="nextTransition"):
         ''' Set a climate hold - ie away, home, sleep '''
-        body = ('{"functions":[{"type":"setHold","params":{"holdType":"'
-                + hold_type + '","holdClimateRef":"' + climate + '"}}],'
-                '"selection":{"selectionType":"thermostats","selectionMatch"'
-                ':"' + self.thermostats[index]['identifier'] + '"}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "setHold", "params": {
+                    "holyType": hold_type,
+                    "holdClimateRef": climate
+                }}]}
         log_msg_action = "set climate hold"
         return self.make_request(body, log_msg_action)
 
     def delete_vacation(self, index, vacation):
         ''' Delete the vacation with name vacation '''
-        body = ('{"functions":[{"type":"deleteVacation","params":{"name":"'
-                + vacation + '"}}],'
-                '"selection":{"selectionType":"registered","selectionMatch":"'
-                '"}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "deleteVacation", "params": {
+                    "name": vacation
+                }}]}
+
         log_msg_action = "delete a vacation"
         return self.make_request(body, log_msg_action)
 
-    def resume_program(self, index, resume_all="false"):
+    def resume_program(self, index, resume_all=False):
         ''' Resume currently scheduled program '''
-        body = ('{"functions":[{"type":"resumeProgram","params":{"resumeAll"'
-                ':"' + resume_all + '"}}],"selection":{"selectionType"'
-                ':"thermostats","selectionMatch":"'
-                + self.thermostats[index]['identifier'] + '"}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "resumeProgram", "params": {
+                    "resumeAll": resume_all
+                }}]}
+
         log_msg_action = "resume program"
         return self.make_request(body, log_msg_action)
 
     def send_message(self, index, message="Hello from python-ecobee!"):
         ''' Send a message to the thermostat '''
-        body = ('{"functions":[{"type":"sendMessage","params":{"text"'
-                ':"' + message[0:500] + '"}}],"selection":{"selectionType"'
-                ':"thermostats","selectionMatch":"'
-                + self.thermostats[index]['identifier'] + '"}}')
+        body = {"selection": {
+                    "selectionType": "thermostats",
+                    "selectionMatch": self.thermostats[index]['identifier']},
+                "functions": [{"type": "sendMessage", "params": {
+                    "text": message[0:500]
+                }}]}
+
         log_msg_action = "send message"
         return self.make_request(body, log_msg_action)
 
     def set_humidity(self, index, humidity):
         ''' Set humidity level'''
-        body = ('{"selection":{"selectionType":"thermostats","selectionMatch":'
-                '"' + self.thermostats[index]['identifier'] +
-                '"},"thermostat":{"settings":{"humidity":"' + str(int(humidity)) +
-                '"}}}')
+        body = {"selection": {"selectionType": "thermostats",
+                              "selectionMatch": self.thermostats[index]['identifier']},
+                              "thermostat": {
+                                  "settings": {
+                                      "humidity": int(humidity)
+                                  }
+                              }}
+
         log_msg_action = "set humidity level"
         return self.make_request(body, log_msg_action)
